@@ -1,39 +1,14 @@
-"""
-The MIT License (MIT)
-
-Copyright (c) 2020-present https://github.com/summer
-
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the "Software"),
-to deal in the Software without restriction, including without limitation
-the rights to use, copy, modify, merge, publish, distribute, sublicense,
-and/or sell copies of the Software, and to permit persons to whom the
-Software is furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-DEALINGS IN THE SOFTWARE.
-"""
-
 import ast
 import base64
-import json
 import logging
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Tuple
 import re
 
 import requests
 
-from .http_client import HTTPClient
-from .models import UserProfile, Profile, Skin, Cape, NameInformation
-from .errors import (
+from mojang.http_client import HTTPClient
+from mojang.types import Profile, Skin, Cape, NameInformation
+from mojang.errors import (
     MojangError,
     BadRequest,
     LoginFailure,
@@ -64,11 +39,11 @@ class MojangAuth(HTTPClient):
         password: Optional[str] = None,
         bearer_token: Optional[str] = None,
         session: Optional[requests.Session] = None,
-        retry_on_rate_limit: Optional[bool] = False,
+        retry_on_ratelimit: Optional[bool] = False,
         ratelimit_sleep_time: Optional[int] = 60,
         debug_mode: Optional[bool] = False,
     ):
-        super().__init__(session, retry_on_rate_limit, ratelimit_sleep_time, debug_mode)
+        super().__init__(session, retry_on_ratelimit, ratelimit_sleep_time, debug_mode)
 
         self.email = email
         self.password = password
@@ -76,19 +51,15 @@ class MojangAuth(HTTPClient):
 
         if bearer_token:
             self._set_authorization_header(bearer_token)
-        elif not email and not password:
-            raise TypeError(
-                "Either an email/password or bearer token must be supplied."
-            )
+        elif email is None and password is None:
+            raise TypeError("Either an email/password or bearer token must be supplied.")
         else:
             self._login()
 
         self._validate_session()
 
-    def _validate_session(self):
-        resp = self.request(
-            "get", f"{_BASE_API_URL}/entitlements/mcstore", ignore_codes=[401]
-        )
+    def _validate_session(self) -> None:
+        resp = self.request("get", f"{_BASE_API_URL}/entitlements/mcstore", ignore_codes=[401])
 
         # The response content is empty if the authorization token isn't set or valid
         if not resp.text:
@@ -113,7 +84,7 @@ class MojangAuth(HTTPClient):
 
         self.session.headers.update({"Authorization": f"{bearer_token}"})
 
-    def _get_oauth2_token_and_url(self):
+    def _get_oauth2_token_and_url(self) -> Tuple[str, str]:
         """Begins the Microsoft OAuth2 Flow"""
         params = {
             "client_id": "000000004C12AE6F",
@@ -124,9 +95,7 @@ class MojangAuth(HTTPClient):
             "locale": "en",
         }
 
-        resp = self.request(
-            "get", "https://login.live.com/oauth20_authorize.srf", params=params
-        )
+        resp = self.request("get", "https://login.live.com/oauth20_authorize.srf", params=params)
 
         # Parses the values via regex since the HTML can't be parsed
         value = re.search(r'value="(.+?)"', resp.text)[0].replace('value="', "")[:-1]
@@ -134,7 +103,7 @@ class MojangAuth(HTTPClient):
 
         return value, url
 
-    def _authenticate_with_microsoft(self, token: str, url: str):
+    def _authenticate_with_microsoft(self, token: str, url: str) -> Tuple[str, str]:
         """Authenticates with Microsoft"""
         payload = {
             "login": self.email,
@@ -155,7 +124,7 @@ class MojangAuth(HTTPClient):
 
         return access_token, refresh_token
 
-    def _authenticate_with_xboxlive(self, access_token: str):
+    def _authenticate_with_xboxlive(self, access_token: str) -> Tuple[str, str]:
         """Authenticates with XBL"""
         json_data = {
             "Properties": {
@@ -167,16 +136,14 @@ class MojangAuth(HTTPClient):
             "TokenType": "JWT",
         }
 
-        resp = self.request(
-            "post", "https://user.auth.xboxlive.com/user/authenticate", json=json_data
-        )
+        resp = self.request("post", "https://user.auth.xboxlive.com/user/authenticate", json=json_data)
 
         xbl_token = resp.json()["Token"]
         user_hash = resp.json()["DisplayClaims"]["xui"][0]["uhs"]
 
         return xbl_token, user_hash
 
-    def _get_xsts_token(self, xbl_token):
+    def _get_xsts_token(self, xbl_token: str) -> str:
         """Gets the XSTS token which is required to authenticate with Minecraft services"""
         json_data = {
             "Properties": {"SandboxId": "RETAIL", "UserTokens": [xbl_token]},
@@ -187,7 +154,7 @@ class MojangAuth(HTTPClient):
         resp = self.request(
             "post",
             "https://xsts.auth.xboxlive.com/xsts/authorize",
-            ignore_codes=401,
+            ignore_codes=[401],
             json=json_data,
         )
 
@@ -232,20 +199,6 @@ class Client(MojangAuth):
 
         Returns:
             A `Profile` object that contains information about a Minecraft profile
-
-        Example:
-            ```py
-            profile = client.get_profile()
-
-            print(profile.id)
-            print(profile.name)
-
-            for skin in profile.skins:
-                print(skin.id)
-                print(skin.enabled)
-                print(skin.url)
-                print(skin.variant)
-            ```
         """
         data = self.request("get", f"{_BASE_API_URL}/minecraft/profile").json()
 
@@ -287,21 +240,8 @@ class Client(MojangAuth):
             A dictionary object that contains information about the account's username. \
                 Possible keys are `changed_at`, `created_at`, \
                 and `name_change_allowed`.
-
-        Example:
-            ```py
-            name_obj = client.get_name_change_info()
-
-            if name_obj.name_change_allowed:
-                print("A name change is allowed")
-
-            print(name_obj.changed_at)
-            print(name_obj.created_at)
-            ```
         """
-        data = self.request(
-            "get", f"{_BASE_API_URL}/minecraft/profile/namechange"
-        ).json()
+        data = self.request("get", f"{_BASE_API_URL}/minecraft/profile/namechange").json()
 
         return NameInformation(
             changed_at=data.get("changedAt"),
@@ -322,21 +262,19 @@ class Client(MojangAuth):
 
         Returns:
             `True` if the username is available; `False` if the username is invalid or already taken
-
-        Example:
-            ```py
-            if client.is_username_available("Notch"):
-                print("The username Notch is available.")
-            else:
-                print("The username Notch is not available.")
-            ```
         """
+        if len(username) < 2 or len(username) > 16:
+            raise ValueError("Username size must be between 3 and 16 characters")
 
-        resp = self.request(
-            "get", f"{_BASE_API_URL}/minecraft/profile/name/{username}/available"
-        )
+        resp = self.request("get", f"{_BASE_API_URL}/minecraft/profile/name/{username}/available")
 
-        return bool(resp.json().get("name"))
+        data = resp.json()
+
+        if data.get("status"):
+            if data["status"] == "AVAILABLE":
+                return True
+
+        return False
 
     def is_username_blocked(self, username: str) -> bool:
         """
@@ -355,9 +293,11 @@ class Client(MojangAuth):
         Returns:
             `True` if the username is blocked; `False` if the username is not blocked
         """
-        resp = self.request(
-            "get", f"{_BASE_API_URL}/minecraft/profile/name/{username}/available"
-        )
+
+        if len(username) < 2 or len(username) > 16:
+            raise ValueError("Username size must be between 3 and 16 characters")
+
+        resp = self.request("get", f"{_BASE_API_URL}/minecraft/profile/name/{username}/available")
 
         data = resp.json()
 
@@ -381,15 +321,6 @@ class Client(MojangAuth):
             A dictionary object that contains information about whether the username was claimed. \
                 Possible keys are `success` (which contains `True` or `False`) and `error` with an error message \
                 (only if the function fails).
-
-        Example:
-            ```py
-            data = client.change_username("Notch")
-            if not data["success"]:
-                print(data["error"])
-            elif data["success"]:
-                print("The username Notch has successfully been claimed.")
-            ```
         """
 
         resp = self.request(
@@ -403,11 +334,9 @@ class Client(MojangAuth):
 
         if resp.status_code == 400:
             try:
-                error = resp.json()["errorMessage"].replace(
-                    "changeProfileName.profileName:", ""
-                )
-            except (requests.JSONDecodeError, KeyError):
-                raise BadRequest(response=resp)
+                error = resp.json()["errorMessage"].replace("changeProfileName.profileName:", "")
+            except (requests.JSONDecodeError, KeyError) as exc:
+                raise BadRequest(response=resp) from exc
 
             return dict(success=False, error=error)
 
@@ -439,43 +368,24 @@ class Client(MojangAuth):
 
         Raises:
             MojangError: If the skin could not be changed for some reason.
-
-        Example:
-            ```py
-            # Change your skin via URL
-            skin_url = "http://textures.minecraft.net/texture/2ff6d970b1b6243fe5a44c5ac540c320506987a5c55ba99a90f758b00d3e05a1"
-            client.change_skin(variant="slim", url=skin_url)
-
-            # Change your skin via file path / image name
-            client.change_skin(variant="classic", image_path="skin.png")
-            ```
         """
         variant = variant.strip().lower()
 
         if variant != "slim" and variant != "classic":
             raise ValueError("Skin variant must be set to either slim or classic.")
-        elif not image_path and not url:
-            raise TypeError(
-                "Missing required parameters. Please supply a skin URL or a skin image path."
-            )
+
+        if image_path is None and url is None:
+            raise TypeError("Missing required parameters. Please supply a skin URL or a skin image path.")
 
         if url:
             json_payload = {"url": url, "variant": variant}
-            resp = self.request(
-                "post", f"{_BASE_API_URL}/minecraft/profile/skins", json=json_payload
-            )
+            self.request("post", f"{_BASE_API_URL}/minecraft/profile/skins", json=json_payload)
         else:
             files = {
                 "file": open(f"{image_path}", "rb"),
                 "variant": (None, variant),
             }
-            resp = self.request(
-                "post", f"{_BASE_API_URL}/minecraft/profile/skins", files=files
-            )
-
-        if not resp.ok:
-            error_message = resp.json()["errorMessage"]
-            raise MojangError(error_message)
+            self.request("post", f"{_BASE_API_URL}/minecraft/profile/skins", files=files)
 
     def copy_skin(
         self,
@@ -492,58 +402,49 @@ class Client(MojangAuth):
 
         Raises:
             MojangError: If an invalid username or UUID is supplied.
-
-        Example:
-            ```py
-            # Copy Notch's skin
-            client.copy_skin("Notch")
-            ```
         """
-        if not username and not uuid:
+        if username is None and uuid is None:
             raise TypeError("Either a username or a UUID must be supplied")
         if username:
-            resp = self.request(
-                "get", f"https://api.mojang.com/users/profiles/minecraft/{username}"
-            )
+            resp = self.request("get", f"https://api.mojang.com/users/profiles/minecraft/{username}")
             try:
                 uuid = resp.json()["id"]
-            except json.decoder.JSONDecodeError:
-                raise MojangError("Username does not exist")
+            except requests.JSONDecodeError as exc:
+                raise MojangError("Username does not exist") from exc
 
         resp = self.request(
-            "get", f"https://sessionserver.mojang.com/session/minecraft/profile/{uuid}"
+            "get",
+            f"https://sessionserver.mojang.com/session/minecraft/profile/{uuid}",
+            ignore_codes=[400],
         )
 
         try:
             value = resp.json()["properties"][0]["value"]
-        except (KeyError, json.decoder.JSONDecodeError):
-            raise MojangError("Invalid UUID supplied")
-
+        except (KeyError, requests.JSONDecodeError) as exc:
+            raise MojangError("Invalid UUID supplied") from exc
         data = ast.literal_eval(base64.b64decode(value).decode())
-        profile = UserProfile(data)
 
-        if not profile.skin_url:
-            raise MojangError("User does not have a skin")
+        skin_url = None
+        skin_variant = "classic"
+        if data["textures"].get("SKIN"):
+            skin_url = data["textures"]["SKIN"]["url"]
+            if data["textures"]["SKIN"].get("metadata"):
+                skin_variant = "slim"
 
-        self.change_skin(
-            url=profile.skin_url,
-            variant=profile.skin_variant,
-        )
+        # If the user doesn't have a skin, also reset the player's skin back to the default
+        if skin_url is None:
+            self.reset_skin()
+        else:
+            self.change_skin(
+                url=skin_url,
+                variant=skin_variant,
+            )
 
     def change_skin_variant(self, variant: str) -> None:
         """Change the skin variant for your current Minecraft skin.
 
         Args:
             variant: Set "slim" for the slim model, or "classic" for the default.
-
-        Example:
-            ```py
-            # Change your skin model to classic
-            client.change_skin_variant("classic")
-
-            # Change your skin model to slim
-            client.change_skin_variant("slim")
-            ```
         """
         profile = self.get_profile()
         self.change_skin(url=profile.skins[0].url, variant=variant)
