@@ -1,27 +1,3 @@
-"""
-The MIT License (MIT)
-
-Copyright (c) 2020-present https://github.com/summer
-
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the "Software"),
-to deal in the Software without restriction, including without limitation
-the rights to use, copy, modify, merge, publish, distribute, sublicense,
-and/or sell copies of the Software, and to permit persons to whom the
-Software is furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-DEALINGS IN THE SOFTWARE.
-"""
-
 import time
 from typing import Any, List, Optional
 import logging
@@ -31,7 +7,7 @@ import requests
 from http.client import HTTPConnection
 
 
-from .errors import (
+from mojang.errors import (
     MojangError,
     BadRequest,
     Forbidden,
@@ -48,12 +24,12 @@ class HTTPClient:
     def __init__(
         self,
         session: Optional[requests.Session] = None,
-        retry_on_rate_limit: Optional[bool] = False,
+        retry_on_ratelimit: Optional[bool] = False,
         ratelimit_sleep_time: Optional[int] = 60,
         debug_mode: Optional[bool] = False,
     ):
         self.ratelimit_sleep_time = ratelimit_sleep_time
-        self.retry_on_rate_limit = retry_on_rate_limit
+        self.retry_on_ratelimit = retry_on_ratelimit
 
         if session:
             self.session = session
@@ -108,10 +84,8 @@ class HTTPClient:
             raise NotFound
 
         if resp.status_code == 429:
-            if self.retry_on_rate_limit:
-                log.warning(
-                    f"We are being ratelimited. Sleeping for {self.ratelimit_sleep_time} seconds."
-                )
+            if self.retry_on_ratelimit:
+                log.warning(f"We are being ratelimited. Sleeping for {self.ratelimit_sleep_time} seconds.")
                 time.sleep(self.ratelimit_sleep_time)
                 return self.request(method, url, ignore_codes, **kwargs)
             else:
